import { ParseAssertion, ParseMatch } from "./builderHelpers";
/**
 * Build a regular expression.
 * @param callback {ExpressionBuilderCallback} - is a callback where the expression is created in.
 * @param options {ExpressionBuilderOptions} - are the options for the expression builder.
 */
export var ExpressionBuilder = function (callback, options) {
    var expression = "";
    var start = "";
    var end = "";
    callback({
        Match: function (character) {
            expression += ParseMatch(character);
        },
        Assert: function (assertion) {
            expression += ParseAssertion(assertion);
        },
        PositiveLookahead: function (callback) {
            var temp = expression;
            expression = "";
            callback();
            end += "(?=" + expression + ")";
            expression = temp;
        },
        NegativeLookahead: function (callback) {
            var temp = expression;
            expression = "";
            callback();
            end += "(?!" + expression + ")";
            expression = temp;
        },
        PositiveLookbehind: function (callback) {
            var temp = expression;
            expression = "";
            callback();
            start += "(?<=" + expression + ")";
            expression = temp;
        },
        NegativeLookbehind: function (callback) {
            var temp = expression;
            expression = "";
            callback();
            start += "(?<!" + expression + ")";
            expression = temp;
        },
        OR: function () {
            var callbacks = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                callbacks[_i] = arguments[_i];
            }
            var temp = expression;
            expression = "";
            var values = [];
            for (var _a = 0, callbacks_1 = callbacks; _a < callbacks_1.length; _a++) {
                var callback_1 = callbacks_1[_a];
                callback_1();
                values.push("(?:" + expression + ")");
                expression = "";
            }
            expression = temp + "(?:" + values.join("|") + ")";
        },
        InSet: function (callback) {
            var temp = expression;
            expression = "";
            callback();
            expression = temp + "[" + expression + "]";
        },
        NotInSet: function (callback) {
            var temp = expression;
            expression = "";
            callback();
            expression = temp + "[^" + expression + "]";
        },
        To: function () {
            expression += "-";
        },
        Group: function (callback) {
            var temp = expression;
            expression = "";
            callback();
            expression = temp + "(" + expression + ")";
        },
        BackRef: function (index) {
            expression += "\\" + index;
        },
        Repeat: function (callback, options) {
            var temp = expression;
            expression = "";
            callback();
            expression = temp + "(?:" + expression + options.repeatChars + ")";
        }
    });
    return "/" + start + expression + end + "/";
};
/**
 * Special characters that can be used in matches.
 */
export var Characters;
(function (Characters) {
    Characters[Characters["ANY"] = 0] = "ANY";
    Characters[Characters["DIGIT"] = 1] = "DIGIT";
    Characters[Characters["NON_DIGIT"] = 2] = "NON_DIGIT";
    Characters[Characters["ALPHANUM"] = 3] = "ALPHANUM";
    Characters[Characters["NON_ALPHANUM"] = 4] = "NON_ALPHANUM";
    Characters[Characters["SPACE"] = 5] = "SPACE";
    Characters[Characters["NON_SPACE"] = 6] = "NON_SPACE";
    Characters[Characters["HTAB"] = 7] = "HTAB";
    Characters[Characters["VTAB"] = 8] = "VTAB";
    Characters[Characters["RETURN"] = 9] = "RETURN";
    Characters[Characters["LINEFEED"] = 10] = "LINEFEED";
    Characters[Characters["FORMFEED"] = 11] = "FORMFEED";
    Characters[Characters["BACKSPACE"] = 12] = "BACKSPACE";
    Characters[Characters["NULL"] = 13] = "NULL";
})(Characters || (Characters = {}));
(function (Characters) {
    /**
     * Match a control character.
     * @param caret {string} - is a one-length character from A-Z indicating the control character.
     */
    Characters.Control = function (caret) {
        if (caret.length !== 1 || !/[A-Z]/.test(caret.toUpperCase()))
            throw new Error("The caret must be a character in A-Z.");
        return LiteralCharacter("\\c" + caret.toUpperCase());
    };
    /**
     * Match a character by its character code
     * @param hex {string} - is a two-length or four-length string of hexadecimal digits indicating the character code.
     */
    Characters.Hex = function (hex) {
        if (![2, 4].includes(hex.length) || !/^[0-9a-fA-F]{2}$/.test(hex))
            throw new Error("The hex must be two or four characters in 0-9 or A-F");
        return LiteralCharacter((hex.length === 2 ? "\\x" : "\\u") + hex.toLowerCase());
    };
})(Characters || (Characters = {}));
/**
 * Match a character literally, without escaping.
 * @param character {string} - the character being matched.
 */
export var LiteralCharacter = function (character) {
    return { character: character };
};
/**
 * An assertion.
 */
export var Assertion;
(function (Assertion) {
    Assertion[Assertion["START"] = 0] = "START";
    Assertion[Assertion["END"] = 1] = "END";
    Assertion[Assertion["WORD_BOUNDARY"] = 2] = "WORD_BOUNDARY";
    Assertion[Assertion["NOT_WORD_BOUNDARY"] = 3] = "NOT_WORD_BOUNDARY";
})(Assertion || (Assertion = {}));
/**
 * Options for a repeat.
 */
export var RepeatOptions;
(function (RepeatOptions) {
    /**
     * Repeat zero or more times.
     * @param greedy {boolean} - if set to true, will match as many times as possible, otherwise as least times as possible.
     */
    RepeatOptions.ZeroOrMore = function (greedy) {
        if (greedy === void 0) { greedy = true; }
        return { repeatChars: "*" + (greedy ? "" : "?") };
    };
    /**
     * Repeat one or more times.
     * @param greedy {boolean} - if set to true, will match as many times as possible, otherwise as least times as possible.
     */
    RepeatOptions.OneOrMore = function (greedy) {
        if (greedy === void 0) { greedy = true; }
        return { repeatChars: "+" + (greedy ? "" : "?") };
    };
    /**
     * Repeat zero or one times.
     */
    RepeatOptions.ZeroOrOne = function () {
        return { repeatChars: "?" };
    };
    /**
     * Repeat exactly some amount of times.
     * @param count {number} - the amount of times to repeat.
     */
    RepeatOptions.Exactly = function (count) {
        return { repeatChars: "{" + count + "}" };
    };
    /**
     * Repeat at least some amount of times.
     * @param count {number} - the minimum amount of times to repeat.
     */
    RepeatOptions.AtLeast = function (count) {
        return { repeatChars: "{" + count + ",}" };
    };
    /**
     * Repeat between some amount of times and another amount of times (inclusive).
     * @param count1 {number} - the lower bounds for the amount of times to repeat.
     * @param count2 {number} - the upper bounds for the amount of times to repeat.
     */
    RepeatOptions.Between = function (count1, count2) {
        return { repeatChars: "{" + count1 + "," + count2 + "}" };
    };
})(RepeatOptions || (RepeatOptions = {}));
//# sourceMappingURL=builder.js.map